#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Shirley Dango of Rational Software.
# It was created on December 6, 2002 for the RATLC production database as part of the CC/CQ/UCM Integration project.
# The purpose is to update the Build record in the RATLC production database.
# The script will search for the build record within the RATLC production database.
# The user must provide the user login id, password, database name (RATLC), database set name,
# the build record to update by using the build name or build ID, and the new promotion level information.
#
# Update note: This script currently has an optional parameter for the build start time.  At this time, the record
# cannot be updated with this information.  If the parameter is used and a value passed in, the record
# is not updated with the information. 
######################################################################################################################
# 
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003
# Updated January 2008, by Katie Carpenter
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	id			The id of the BTBuild record to update
#	buildstatus		Specifies what the status of the build.  If it passed or failed.
#	buildstarttime		Time of day build was started.
#	buildendtime		Time of day build completed.
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
# use Date::Manip qw(ParseDate UnixDate);

my $progname = basename $0;
my ($entity);
my ($failure,$status); #thl2
my $CRList;
my $item;
my $slashvob;
my $noslashvob;
my $buildtype;
my $securitypolicy;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
    print "

    ʹ CQ API ¼Ựʵó
    ÷
        $progname
 	-id RATLC000xxxxx
 	-user <û>
 	-pw <>
 	-dbname <ݿ>
 	-dbset <ݿ⼯>
 	-pvob<pvob>
 	-baseline<>
	-buildstatus<״̬>
	-securitypolicy<ALM ȫ>
	-buildtype<>
    -buildstarttime<ʼʱ>
    -buildendtime<ʱ>
    -logfile
    -test
    -help		
";
return 0;
}

sub main {	
####
# Define the input parameters
my @opts = qw(
    id=s
    user=s	
    pw=s		
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    buildstatus:s
    securitypolicy=s
    buildtype=s
    buildstarttime:s
    buildendtime:s
    state=s
    logfile=s
    test
    help
);

my %opts = (
    id => undef,
    user => undef, 
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    securitypolicy=> undef,
    buildtype=> undef,
    logfile =>undef,
    state => "Modify", 
);

####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
    $logger->log_print_message(__LINE__ . "ʾ÷");	
     my $retval = &Usage;
     return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error(__LINE__ . "޷־ļĿ¼$logdir");
            		$logger->log_print_error(__LINE__ . "űڲĿ¼¼ִС");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error(__LINE__ . "޷־ļ$opts{logfile}");
        	$logger->log_print_error(__LINE__ . "űڲĿ¼¼ִС");
   	 }
        $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("¼ $log_time ʼ"); 
}	 

####
# Display the help usage
if ($opts{help}) {
 
   	print "г˱Ĳ\n";     
        print "-user		          ClearQuest ûĵ¼ʶ\n";
	print "-pw		            ClearQuest û롣\n";
        print "-dbname		        ݿģʽ洢ӡ\n";
	print "-dbset		         ûݿӡ\n";
	print "-logfile		       ־ļ͵·\n";
        print "-pvob		          Ŀ VOB ơ\n";
	print "-baseline  	     ߵơ\n";
	print "-securitypolicy	 ¼ ALM ȫԡ\n";
	print "-buildtype       ͡\n";
	print "г˿ѡĲ\n";
	print "-state <״̬>    ϣ¼״̬\n";	
	print "-buildstatus		   ָ״̬ͨʧܡ\n";
	print "-buildstarttime		ʼʱ䡣\n";
	print "-buildendtime		  ɹʱ䡣\n";
	print "-test		          ΪļõĿѡ־\n";
	print "-help            ڰĿѡ־\n";
    return 1;
}

# Display test inputs to log file 
if ($opts{test}){
$logger->log_print_message(__LINE__ . "Ѷû룺");
	$logger->log_print_message(__LINE__ . "ѶĹ¼ʶΪ $opts{id}");
	$logger->log_print_message(__LINE__ . "Ѷû¼Ϊ $opts{user}");
	$logger->log_print_message(__LINE__ . "Ѷݿ⼯Ϊ $opts{dbset}");
	$logger->log_print_message(__LINE__ . "ѶݿΪ $opts{dbname}");
	$logger->log_print_message(__LINE__ . "ѶĻΪ $opts{baseline}");
	$logger->log_print_message(__LINE__ . "Ѷ״̬Ϊ $opts{state}");
	$logger->log_print_message(__LINE__ . "ѶĿ VOB Ϊ $opts{pvob}");
	$logger->log_print_message(__LINE__ . "ѶĹΪ $opts{buildtype}");
	$logger->log_print_message(__LINE__ . "ѶĲƷΪ $opts{product}");
	$logger->log_print_message(__LINE__ . "ѶİȫΪ $opts{securitypolicy}");
	if ($opts{logfile}){ $logger->log_print_message("Ѷ־ļΪ $opts{logfile}");}
	if ($opts{buildstatus}){ $logger->log_print_message("ѶĹ״̬Ϊ $opts{buildstatus}");}
	if ($opts{buildstarttime}){ $logger->log_print_message("ѶĹʼʱΪ $opts{buildstarttime}");}
	if ($opts{buildendtime}){ $logger->log_print_message("ѶĹʱΪ $opts{buildendtime}");}
	return 1;
}
####
# End program if any of the required parameters are not defined


if (!defined $opts{user}
  or !defined $opts{pw} 
  or !defined $opts{dbname} 
  or !defined $opts{dbset} 
  or !defined $opts{id}
  or !defined $opts{pvob} 
  or !defined $opts{securitypolicy}
  or !defined $opts{buildtype}) {
  $logger->log_print_message(__LINE__ . "ʾ÷");
  my $retval = &Usage();
  return 0;
}
;
####
# Start a CQ session
print (" CQ Ự\n") ;
my $session = CQPerlExt::CQSession_Build();
####
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error(__LINE__ . "main: ¼ $opts{dbname} ʧܣ$@"); 
return 0;
}

$logger->log_print_message(__LINE__ . "ѳɹ¼ $opts{dbname}");


#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}
($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});

#### Build a query to find the Build record based upon Build ID
my $record;
eval {$record = $session->GetEntity("BTBuild", $opts{id})};
if ($@) {
	$logger->log_print_error(__LINE__ . "main: δҵ빹¼ʶ $opts{id} ƥļ¼"); 
	$logger->log_print_error(__LINE__ . "main:  create_build_record.pl ԴĹ¼"); 
		return 0;
}
	{
	#### Print out the resultset
	$logger->log_print_message(__LINE__ . "ҵ¼$opts{id}");
	
	####Get record 
	# Add updates to the record
	$session->EditEntity($record,"$opts{state}");
	$logger->log_print_message(__LINE__ . " $opts{state} ¼$opts{id}");
	 	
        #Set required values

	my $baselinename = $noslashvob . ' ' . $opts{baseline};
	
	$record->SetFieldValue("ALMBaseline", $baselinename);
		$logger->log_print_message(__LINE__ . " ALMBaseline$baselinename");
	$record->SetFieldValue("ALMBuildType", $opts{buildtype});
		$logger->log_print_message(__LINE__ . " ALMBuildType$opts{buildtype}");
	$record->SetFieldValue("ALMOwner", $opts{user});
		$logger->log_print_message(__LINE__ . " ALMOwner$opts{user}");
	$record->SetFieldValue("ALMSecurityPolicy", $opts{securitypolicy});
		$logger->log_print_message(__LINE__ . " ALMSecurityPolicy$opts{securitypolicy}");	

	#Results from Building
	if ($opts{buildstatus}ne ""){
		$record->SetFieldValue("ALMBuildStatus", $opts{buildstatus});
		$logger->log_print_message(__LINE__ . " BuildStatus$opts{buildstatus}");
	}
	
	if ($opts{buildstarttime} ne ""){
		$record->SetFieldValue("Start_DateTime", $opts{buildstarttime});
		$logger->log_print_message(__LINE__ . " BuildStartTime$opts{buildstarttime}");
	}
	if ($opts{buildendtime} ne ""){
		$record->SetFieldValue("End_DateTime", $opts{buildendtime});
		$logger->log_print_message(__LINE__ . " BuildEndTime$opts{buildendtime}");
	}
		
	
	
	$logger->log_print_message(__LINE__ . "֤¼ $opts{id}");
	$status = "" ; #thl2
	eval {$status = $record->Validate()};
	if (($@) || ($status)) {  #thl2
		$logger->log_print_error(__LINE__ . "޷֤ ClearQuest ݿеĹ¼$status");
	}
	if ($@) {
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: ޷֤ ClearQuest ݿеĹ¼");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ$status"); #thl2
		return 0;
	}
	$status = "" ; #thl2
	$logger->log_print_message(__LINE__ . "ʵ¼ $opts{id}");
	eval {$status = $record->Commit()};
	if (($@) || ($status)) {  #thl2
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: ޷ ClearQuest ݿʵ¼");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ$status"); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
	$record = undef;
	$logger->log_print_success(__LINE__ . "ѳɹ $opts{id}");
	CQPerlExt::CQSession_Unbuild($session);
	return 1;
	} #end block
$logger->log_print_success(" Update_build_record ű");
$logger->log_print_message(__LINE__ . "Update_build_record ű");
CQPerlExt::CQSession_Unbuild($session);
return 1;
} #end sub main
#### End of Script
#### SUBROUTINES FOR PROGRAM
